{% extends "_page_base.html" %}

{% block title %}
Bokeh Plot Server &mdash; {{ title }}
{% endblock %}

{% block script %}
Bokeh.embed.server_page("{{ title }}");
{% endblock %}

{% block content %}
<div class="col-md-12">
  <div id="PlotPane"></div>
</div>
{% endblock %}
