{% extends "_page_base.html" %}

{% block title %}
Bokeh Plot Server &mdash; {{ title }}
{% endblock %}

{% block extra_scripts %}
  {% for modulename, classname, parentname in extra_generated_classes %}
  <script src="{{url_for('bokeh.server.generatejs', modulename=modulename, classname=classname, parentname=parentname) }}"></script>
  {% endfor %}
{% endblock %}

{% block script %}
window.Bokeh = Bokeh;
{% for modulename, classname, parentname in extra_generated_classes %}
window.{{modulename}}.main();
{% endfor %}
Bokeh.one_object_page("{{docid}}", "{{objid}}")
{% endblock %}

{% block content %}
<div class="col-md-12">
  <div id="PlotPane"></div>
</div>
{% endblock %}
