{% extends "_base.html" %}

{% block body %}
  <div class="container">
    {% from "_navbar.html" import navbar %}
    {% call navbar(title) %}
      <li><a href="{{ url_for('bokeh.server.register_get') }}">register</a></li>
      <li><a href="{{ url_for('bokeh.server.login_get') }}">login</a></li>
      <li><a href="{{ url_for('bokeh.server.logout') }}">logout</a></li>
    {% endcall %}
    <div class="row">
      <div class="col-md-12">
        <div class="messages">
          {% with messages = get_flashed_messages(with_categories=true) %}
            {% if messages %}
              {% for category, message in messages %}
              <div class="alert alert-{{ category }}">
                <button type="button" class="close" data-dismiss="alert">&times;</button>
                {{ message }}
              </div>
              {% endfor %}
            {% endif %}
          {% endwith %}
        </div>
      </div>
    </div>
    {% block main %}
    {% endblock %}
  </div>
{% endblock %}
