{% extends "_auth_base.html"%}

{% block main %}
<form role="form" action="/bokeh/register" method="POST">
  <div class="form-group">
    <label for="username">username</label>
    <input type="text" name="username" class="form-control" />
  </div>
  <div class="form-group">
    <label for="password">password</label>
    <input type="password" name="password" class="form-control" />
  </div>
  <div class="form-group">
    <label for="password_confirm">password confirmation</label>
    <input type="password" name="password_confirm" class="form-control" />
  </div>
  </br>
  <button type="submit" class="btn btn-default">Submit</button>
</form>
{% endblock %}
