<script
    src="{{ src_path }}"
    id="{{ elementid }}"
    async="true"
    data-bokeh-data="static"
    data-bokeh-modelid="{{ modelid }}"
    data-bokeh-modeltype="{{ modeltype }}"
    data-bokeh-loglevel="{{ loglevel }}"
></script>
