#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2014, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
"""
Supporting objects and functions to convert Matplotlib objects into Bokeh.

"""

from .compat.mpl import  to_bokeh; to_bokeh
